package de.jpv.openinidea.client;

/*
 *  Copyright 2007 Jens Vo.
 *
 *  Licensed under the Apache License version 2.0 (the "License").
 *  You may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

import java.io.File;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.UnsupportedEncodingException;
import java.net.Socket;
import java.net.URLDecoder;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;
import javax.swing.JOptionPane;

import de.jpv.openinidea.shared.Config;
import de.jpv.openinidea.shared.Request;


/**
 * The <code>Client</code> is a standalone Java application whose
 * <code>main()</code> method will attempt to establish a communication with
 * the corresponding <code>de.jpv.openinidea.server.Server</code> (which is
 * a <code>Thread</code> within the OpenInIDEA plugin) and ask it to open a
 * specified file.
 */
public class Client {

  private static String getResponse(final Request req)
          throws IOException, ClassNotFoundException {
    ObjectOutputStream out = null;
    ObjectInputStream in = null;
    Socket socket = null;
    try {
      socket = new Socket("localhost", Config.getPort());
      out = new ObjectOutputStream(socket.getOutputStream());
      out.writeObject(req);
      in = new ObjectInputStream(socket.getInputStream());
      return (String) in.readObject();
    }
    finally {
      if (out != null) {
        out.close();
      }
      if (in != null) {
        in.close();
      }
      if (socket != null) {
        socket.close();
      }
    }
  }

  private static List<String> extractProjectUrls(final String response)
          throws UnsupportedEncodingException {
    final ArrayList<String> retVal = new ArrayList<String>();
    for (StringTokenizer tok = new StringTokenizer(response, "|", false);
         tok.hasMoreTokens();) {
      retVal.add(URLDecoder.decode(tok.nextToken(), "UTF-8"));
    }
    return retVal;
  }

  private static void exit(int exitCode, String errorMessage) {
    if (errorMessage != null) {
      System.out.println(errorMessage);
      JOptionPane.showMessageDialog(null, errorMessage, "OpenInIDEA",
                                    JOptionPane.ERROR_MESSAGE);

    }
    System.exit(exitCode);
  }

  private static void selectProject(Request req, String res)
          throws UnsupportedEncodingException, InterruptedException {
    List<String> projects = extractProjectUrls(res);
    ProjectSelector selector = new ProjectSelector(projects);
    selector.show(null, 500, 300);
    int selection;
    do {
      selection = selector.getSelection();
      Thread.sleep(50L);
    }
    while (selection == -1);
    req.setProjectUrl(projects.get(selection));
  }

  /**
   * The <code>Client</code>'s <code>main</code> method expects the absolute
   * path of the file to be opened in IntelliJ IDEA as its only argument.
   * When invoked, it will communicate that file name to the corresponding
   * {@link de.jpv.openinidea.server.Server} to be opened; if IDEA has more
   * than one project opened, it will come back to the user (via a popup menu)
   * and ask for the specification of the project in which the file is to be
   * opened.<br/>
   * In the case of an error (e.g. if the specified file does not exists, or
   * the connection with the server cannot be established, or the server is
   * unable to open the file (e.g. because it does not recognize the file
   * type)), the user is informed via a dialog window.
   *
   * @param args The argument array. This array is expected to contain at
   *         least one element specifying absolute path of the file to be
   *         opened. Additional argument array elements will be ignored.
   */
  public static void main(final String[] args) {

    try {

      if (args.length == 0) {
        exit(-1, "No file specified.");
      }
      final File file = new File(args[0]);
      if (!file.exists()) {
        exit(-2, "File " + file.getAbsolutePath() + " does not exist");
      }

      final Request req = new Request(file.getAbsolutePath());
      do {
        String res = getResponse(req);
        if (res == null || res.length() == 0) {
          break;
        }
        if (!res.startsWith("|")) {
          exit(-3, res);
        }
        if (req.getProjectUrl() != null) {
          exit(-4, "Project has been set");
        }
        selectProject(req, res);
      }
      while (true);
    }
    catch (Throwable e) {
      e.printStackTrace();
      exit(-5, e.getMessage());
    }

  }

  private Client() {
  }

}
