package de.jpv.openinidea.server;

/*
 *  Copyright 2007 Jens Vo.
 *
 *  Licensed under the Apache License version 2.0 (the "License").
 *  You may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

import java.io.File;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;

import javax.swing.JComponent;

import com.intellij.diagnostic.PluginException;
import com.intellij.openapi.application.ApplicationManager;
import com.intellij.openapi.fileEditor.FileEditor;
import com.intellij.openapi.fileEditor.FileEditorManager;
import com.intellij.openapi.fileTypes.FileTypeManager;
import com.intellij.openapi.fileTypes.StdFileTypes;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.vfs.LocalFileSystem;
import com.intellij.openapi.vfs.VirtualFile;
import com.intellij.openapi.wm.WindowManager;

/**
 * This class is the "workhorse" of the OpenInIDEA plugin. It encapsulates one
 * attempt to open a <code>FileEditor</code> for a specified <code>File</code>
 * in a specified <code>Project</code>.<br/>
 * Since the action of opening a file is not invoked by the AWT event
 * dispatching thread, the <code>OpenFile</code> class is implementing the
 * <code>Runnable</code> interface so that its <code>run()</code> method can
 * be executed asynchronously via
 * <code>com.intellij.openapi.application.Application.invokeLater()</code>.
 */
public class OpenFile implements Runnable {

  /**
   * The <code>OpenFile.State</code> enum is enumerating the different
   * possible states of an <code>OpenFile</code> object.
   */
  public enum State {

    /**
     * This state indicates that the <code>OpenFile</code> object is still new
     * and has not executed its <code>run()</code> method.
     */
    NEW,
    /**
     * The <code>SUCCESS</code> state indicates that the <code>OpenFile</code>
     * object has executed its <code>run()</code> method and has successfully
     * opened the specified file.
     */
    SUCCESS,
    /**
     * An <code>OpenFile</code> object will have the <code>NOT_FOUND</code>
     * state if the specified file was not found.
     */
    NOT_FOUND,
    /**
     * If an error occurs during the <code>run()</code> method of an
     * <code>OpenFile</code> object, the object will get this state. If the
     * error involves an exception, this exception can be obtained via the
     * {@link OpenFile#getException()} method.
     */
    ERROR
  }

  private State state = State.NEW;
  private Exception exception = null;

  private final Project project;
  private final String fileName;

  /**
   * Public constructor.
   *
   * @param project The <code>Project</code> in which the specified file is
   *         to be opened
   * @param fileName The absolute path of the file to be opened
   */
  public OpenFile(Project project, String fileName) {
    this.project = project;
    this.fileName = fileName;
  }

  /**
   * The <code>run()</code> method performs the attempt to open the specified
   * file. After execution of this method, the <code>OpenFile</code>'s state
   * can be obtained via the {@link #getState()} method; in the case of an
   * exception, this
   */
  public void run() {
    try {
      LocalFileSystem lfs = LocalFileSystem.getInstance();
      VirtualFile vf = lfs.refreshAndFindFileByIoFile(new File(fileName));
      if (vf != null) {
        if ("UNKNOWN".equals(vf.getFileType().getName())) {
          FileTypeManager ftm = FileTypeManager.getInstance();
          final String ext = vf.getExtension();
          ftm.associateExtension(StdFileTypes.PLAIN_TEXT, ext);
        }

        FileEditorManager fem = FileEditorManager.getInstance(project);
        FileEditor[] fileEditors = fem.openFile(vf, true);
        if (fileEditors.length > 0) {
          state = State.SUCCESS;
        }
        else {
          state = State.ERROR;
        }
      }
      else {
        state = State.NOT_FOUND;
      }
    }
    catch (RuntimeException e) {
      state = State.ERROR;
      exception = e;
      throw new PluginException(e, OpenInIdeaPlugin.PLUGIN_ID);
    }
    WindowManager.getInstance().getFrame(project).requestFocus();
  }

  /**
   * This method will delegate the {@link #run()} method to the AWT event
   * dispatching thread.
   */
  public void openFile() {
    ApplicationManager.getApplication().invokeLater(new Runnable() {
      public void run() {
        ApplicationManager.getApplication().runWriteAction(OpenFile.this);
      }
    });
    while (state == State.NEW) {
      try {
        Thread.sleep(100L);
      }
      catch (InterruptedException e) {
        throw new PluginException(e, OpenInIdeaPlugin.PLUGIN_ID);
      }
    }
  }

  /**
   * This method is used to obtain the state of this <code>OpenFile</code>.
   *
   * @return The current state of this <code>OpenFile</code>
   */
  public State getState() {
    return state;
  }

  /**
   * This method will retrieve the <code>Exception</code> that might have
   * occurred while {@link #run()}ning this <code>OpenFile</code>.
   *
   * @return The <code>Exception</code> that occurred during the execution
   *          of this <code>OpenFile</code> (if applicable; <b>null</b>
   *          otherwise)

   */
  public Exception getException() {
    return exception;
  }

}
