package de.jpv.openinidea.server;

/*
 *  Copyright 2007 Jens Vo.
 *
 *  Licensed under the Apache License version 2.0 (the "License").
 *  You may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

import java.io.IOException;

import com.intellij.diagnostic.PluginException;
import com.intellij.ide.plugins.PluginManager;
import com.intellij.openapi.components.ProjectComponent;
import com.intellij.openapi.extensions.PluginId;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;

/**
 * <code>OpenInIdeaPlugin</code> is the main <code>ProjectComponent</code> of
 * the OpenInIDEA plugin and is registered in the plugin's configuration file.
 */
public class OpenInIdeaPlugin implements ProjectComponent {

  static final PluginId PLUGIN_ID = PluginManager.getPluginByClassName(
          OpenInIdeaPlugin.class.getName());

  // start the server
  static {
    Server server;
    try {
      server = new Server();
    }
    catch (IOException e) {
      throw new PluginException(e, PLUGIN_ID);
    }
    server.start();
  }

  private static boolean isWindows() {
    String osName = System.getProperty("os.name");
    return osName != null && osName.startsWith("Windows");
  }

  private final WindowsInstaller installer;

  /**
   * Public constructor. It sets up the {@link WindowsInstaller} member of the
   * plugin in case IDEA is running on a Windows platform.
   */
  public OpenInIdeaPlugin() {
    installer = isWindows() ? new WindowsInstaller() : null;
  }

  /**
   * Empty implementation of the method declared in the
   * <code>ProjectComponent</code> interface.
   */
  public void projectOpened() {
  }

  /**
   * Empty implementation of the method declared in the
   * <code>ProjectComponent</code> interface.
   */
  public void projectClosed() {
  }

  /**
   * This method retrieves the name of the OpenInIDEA plugin's name which is
   * (surprise!) "OpenInIDEA".
   *
   * @return The name of this plugin
   */
  @NonNls
  @NotNull
  public String getComponentName() {
    return "OpenInIDEA";
  }

  /**
   * This implementation checks that the installer is run unless it has been
   * run before and unless the operating system is not Windows.
   */
  public void initComponent() {
    if (installer != null && !installer.isInstalled()) {
      installer.install();
    }
  }

  /**
   * Empty implementation of the method declared in the
   * <code>ProjectComponent</code> interface.
   */
  public void disposeComponent() {
  }

}
