package de.jpv.openinidea.server;

/*
 *  Copyright 2007 Jens Vo.
 *
 *  Licensed under the Apache License version 2.0 (the "License").
 *  You may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.UnsupportedEncodingException;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.URLEncoder;
import java.text.MessageFormat;

import com.intellij.diagnostic.PluginException;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.project.ProjectManager;
import de.jpv.openinidea.shared.Config;
import de.jpv.openinidea.shared.Request;

/**
 * The <code>Server</code> Thread listens on the configured
 * {@link Config#getPort() port} for a {@link Request} and attempts to open
 * files for which such <code>Request</code>s were received.
 */
public class Server extends Thread {

  private static final String MSG_NOT_FOUND = "Error: File {0} not found";

  private static String getResponse(final Request req) throws UnsupportedEncodingException {
    final String fileName = req.getFileName();
    if (fileName == null) {
      return "Error: No file name specified";
    }
    final Project[] openProjects = ProjectManager.getInstance().getOpenProjects();
    Project project = null;
    if (openProjects.length == 1) {
      project = openProjects[0];
    }
    else if (req.getProjectUrl() != null) {
      for (Project p : openProjects) {
        if (req.getProjectUrl().equals(p.getPresentableUrl())) {
          project = p;
        }
      }
    }

    if (project != null) {
      OpenFile openFile = new OpenFile(project, fileName);
      openFile.openFile();
      switch (openFile.getState()) {
        case SUCCESS:      return "";
        case NOT_FOUND:    return MessageFormat.format(MSG_NOT_FOUND, fileName);
        case ERROR:
          Throwable e = openFile.getException();
          return e != null ? "Error: " + e.getMessage() : "Unknown Error";
        default: return "Unknown Error";
      }
    }
    else {
      final StringBuilder msg = new StringBuilder();
      for (Project p : openProjects) {
        msg.append('|').append(URLEncoder.encode(p.getPresentableUrl(), "UTF-8"));
      }
      return msg.toString();
    }
  }

  private ServerSocket serverSocket;

  /**
   * Public constructor. Sets up the server socket which listens on the
   * specified port.
   *
   * @throws IOException If the server socket connection cannot be established
   */
  public Server() throws IOException {
    serverSocket = new ServerSocket(Config.getPort());
  }

  private void handleRequest() {
    Socket socket = null;
    ObjectInputStream in = null;
    ObjectOutputStream out = null;
    try {
      socket = serverSocket.accept();
      in = new ObjectInputStream(socket.getInputStream());
      final Request req = (Request) in.readObject();
      final String res = getResponse(req);
      out = new ObjectOutputStream(socket.getOutputStream());
      out.writeObject(res);
    }
    catch (ClassNotFoundException e) {
      throw new PluginException(e, OpenInIdeaPlugin.PLUGIN_ID);
    }
    catch (UnsupportedEncodingException e) {
      throw new PluginException(e, OpenInIdeaPlugin.PLUGIN_ID);
    }
    catch (IOException e) {
      throw new PluginException(e, OpenInIdeaPlugin.PLUGIN_ID);
    }
    finally {
      try {
        if (in != null) {
          in.close();
        }
        if (out != null) {
          out.close();
        }
        if (socket != null) {
          socket.close();
        }
      }
      catch (IOException e) {
        throw new PluginException(e, OpenInIdeaPlugin.PLUGIN_ID);
      }
    }
  }

  /**
   * The <code>Server</code>'s <code>run()</code> method is listening for
   * client requests in an infinite loop. For every such request, it determines
   * a response (which is simply a <code>String</code>; either the empty
   * <code>String</code> (in case of success) or an appropriate error message)
   * and sends it back to the client.
   */
  @Override
  public void run() {
    try {
      while (true) {
        handleRequest();
        try {
          Thread.sleep(50L);
        }
        catch (InterruptedException e) {
          break;
        }
      }
    }
    finally {
      try {
        serverSocket.close();
      }
      catch (IOException e) {
        throw new PluginException(e, OpenInIdeaPlugin.PLUGIN_ID);
      }
    }
  }

}
