package de.jpv.openinidea.server;

/*
 *  Copyright 2007 Jens Vo.
 *
 *  Licensed under the Apache License version 2.0 (the "License").
 *  You may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.URL;
import java.text.MessageFormat;
import java.util.HashMap;
import java.util.Map;
import javax.imageio.ImageIO;

import com.intellij.diagnostic.PluginException;
import com.intellij.ide.plugins.IdeaPluginDescriptor;
import com.intellij.ide.plugins.PluginManager;
import com.intellij.lang.Language;
import com.intellij.openapi.fileTypes.FileType;
import com.intellij.openapi.fileTypes.FileTypeManager;
import com.intellij.openapi.fileTypes.LanguageFileType;
import net.sf.image4j.codec.ico.ICOEncoder;

/**
 * This class is responsible for printing out various icon files and batch
 * files necessary for registering files of different types in a MS Windows 32
 * Registry.<br/>
 * These files will be placed in a (newly created) <code>etc</code> subdirectory
 * of the base directory of the OpenInIDEA plugin.
 */
public class WindowsInstaller {

  private static final String RESOURCE_NAME = "fileTypes/{0}.png";

  private static final String EDIT_KEY = "reg add \"HKCR\\*\\shell\\IntelliJ IDEA\\Command\"";
  private static final String OPEN_KEY = "reg add \"HKCR\\OpenInIDEA\\shell\\open\\command\"";

  private static final String FILE_SAVE_KEY = "reg save \"HKCR\\.{0}\" \"{0}.hiv\"\n";
  private static final String FILE_RESTORE_KEY = "reg restore \"HKCR\\.{0}\" \"{0}.hiv\"\n";
  private static final String FILE_EDIT_KEY = "reg add \"HKCR\\.{0}\" /ve /d \"{1}\"\n";
  private static final String FILE_ICON_KEY = "reg add \"HKCR\\.{0}\\DefaultIcon\" /ve /d \"{1}\\{0}.ico,0\"\n";
  private static final String FILE_OPEN_KEY = "reg add \"HKCR\\.{0}\\Shell\\Open\\Command\"";

  private static final String JAVA_ARGS = " /ve /d \"\\\"{0}\\\" -cp \\\"{1}\\\" de.jpv.openinidea.client.Client \\\"%%1\\\"\"\n";

  private static final Map<String, String[]> ADDITIONAL_MAPPINGS = new HashMap<String, String[]>();

  static {
    ADDITIONAL_MAPPINGS.put("js",   new String[] {"javaScript", "JavaScript File"});
    ADDITIONAL_MAPPINGS.put("jsp",  new String[] {"jsp", "JSP File"});
    ADDITIONAL_MAPPINGS.put("jspx", new String[] {"jspx", "JSPX File"});
    ADDITIONAL_MAPPINGS.put("form", new String[] {"uiForm", "Form File"});
  }

  private File etcDir;
  private String javaArgsString;

  /**
   * Public constructor.
   */
  public WindowsInstaller() {
    final IdeaPluginDescriptor descriptor = PluginManager.getPlugin(
            OpenInIdeaPlugin.PLUGIN_ID);
    if (descriptor == null) {
      throw new IllegalStateException("Plugin has no descriptor.");
    }
    final File pluginRoot = descriptor.getPath();
    etcDir = new File(pluginRoot, "etc");

    final File javaExecutable = new File(new File(System.getProperty("java.home"), "bin"), "javaw.exe");
    final File classPathBase = new File(new File(pluginRoot, "lib"), "OpenInIDEA.jar");
    javaArgsString = MessageFormat.format(JAVA_ARGS, javaExecutable.getAbsolutePath(), classPathBase.getAbsolutePath());
  }

  /**
   * This method determines whether or not the "installation work" of the
   * <code>WindowsInstaller</code> class has been performed before or not.
   *
   * @return <b>true</b> if the <code>etc</code> subdirectory of the plugin's
   *          base directory already exists; <b>false</b> otherwise
   */
  public boolean isInstalled() {
    return etcDir.exists();
  }

  /**
   * The <code>install</code> method will create the files necessary to
   * integrate the plugin with the Windows operating system. When invoked, this
   * method will
   * <ul>
   *   <li>create an <code>etc</code> subdirectory in the base directory of the
   *     plugin,
   *   <li>create an <code>icons</code> subdirectory of that <code>etc</code>
   *     directory,
   *   <li>for every file type known to IDEA having an identifying file
   *     extension,
   *     <ul>
   *       <li>extract IDEA's file type icon and create an <code>.ico</code>
   *         file for that type in the <code>icons</code> directory,
   *       <li>place a batch file in the <code>etc</code> directory which calls
   *         Windows' <em>reg</em> command in order to create a file association
   *         for that file,
   *     </ul>
   *   <li>create a file named <code>etc\registerEditor.bat</code> which will
   *     call the <em>reg</em> command to create a "global" file associaton
   *     for any file.
   * </ul>
   * All file associations created by this method's batch scripts will invoke
   * the {@link de.jpv.openinidea.client.Client} class (as a standalone Java
   * application).
   */
  public void install() {
    etcDir.mkdir();
    final File iconsDir = new File(etcDir, "icons");
    iconsDir.mkdir();

    final FileType[] types = FileTypeManager.getInstance().getRegisteredFileTypes();
    final ClassLoader loader = this.getClass().getClassLoader();

    for (final FileType type : types) {
      if (!(type instanceof LanguageFileType)
          || type.getDefaultExtension().length() == 0) {
        continue;
      }
      final Language language = ((LanguageFileType) type).getLanguage();
      final String imgName = language.getID().toLowerCase();
      addImageAndRegistration(type.getDefaultExtension(), type.getDescription(),
                              imgName, iconsDir, loader);
    }

    for (final String key : ADDITIONAL_MAPPINGS.keySet()) {
      final String[] fileData = ADDITIONAL_MAPPINGS.get(key);
      addImageAndRegistration(key, fileData[1], fileData[0], iconsDir, loader);
    }

    final File editorRegistrationFile = new File(etcDir, "registerEditor.bat");
    FileOutputStream out = null;
    try {
      out = new FileOutputStream(editorRegistrationFile);
      out.write(EDIT_KEY.getBytes());
      out.write(javaArgsString.getBytes());
      out.write(OPEN_KEY.getBytes());
      out.write(javaArgsString.getBytes());
    }
    catch (FileNotFoundException e) {
      throw new PluginException(e, OpenInIdeaPlugin.PLUGIN_ID);
    }
    catch (IOException e) {
      throw new PluginException(e, OpenInIdeaPlugin.PLUGIN_ID);
    }
    finally {
      try {
        if (out != null) {
          out.close();
        }
      }
      catch (IOException e) {
        throw new PluginException(e, OpenInIdeaPlugin.PLUGIN_ID);
      }
    }

  }

  private void addImageAndRegistration(final String extension, final String description,
                                       final String imgName, final File iconsDir,
                                       final ClassLoader loader) {
    final String resourceName = MessageFormat.format(RESOURCE_NAME, imgName);
    final URL res = loader.getResource(resourceName);
    if (res == null) {
      System.out.println("No resource found for filetype with extension " + extension);
      return;
    }

    final String registrationFileName = MessageFormat.format("register_{0}.bat", extension);
    final File registrationFile = new File(etcDir, registrationFileName);
    final String unregistrationFileName = MessageFormat.format("unregister_{0}.bat", extension);
    final File unregistrationFile = new File(etcDir, unregistrationFileName);
    FileOutputStream out = null;
    try {
      final BufferedImage image = ImageIO.read(res);
      ICOEncoder.write(image, new File(iconsDir, extension + ".ico"));

      final String fileSaveKey = MessageFormat.format(FILE_SAVE_KEY, extension);
      final String fileRestoreKey = MessageFormat.format(FILE_RESTORE_KEY, extension);
      final String fileEditKey = MessageFormat.format(FILE_EDIT_KEY, extension, description);
      final String fileIconKey = MessageFormat.format(FILE_ICON_KEY, extension, iconsDir.getAbsolutePath());
      final String fileOpenKey = MessageFormat.format(FILE_OPEN_KEY, extension);

      out = new FileOutputStream(registrationFile);
      out.write(fileSaveKey.getBytes());
      out.write(fileEditKey.getBytes());
      out.write(fileIconKey.getBytes());
      out.write(fileOpenKey.getBytes());
      out.write(javaArgsString.getBytes());
      out.close();

      out = new FileOutputStream(unregistrationFile);
      out.write(fileRestoreKey.getBytes());
      out.close();
    }
    catch (FileNotFoundException e) {
      throw new PluginException(e, OpenInIdeaPlugin.PLUGIN_ID);
    }
    catch (IOException e) {
      throw new PluginException(e, OpenInIdeaPlugin.PLUGIN_ID);
    }
    finally {
      try {
        if (out != null) {
          out.close();
        }
      }
      catch (IOException e) {
        throw new PluginException(e, OpenInIdeaPlugin.PLUGIN_ID);
      }
    }
  }

}
